CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCYCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      program cubic_cluster
C
C  QMC main program of cubic cluster with infinite walls
C
       use highlevel
       use midlevel
       use orbital
       use determinant
       use random
       use jastrow, JEXP=>JASEXPATOM
       use observables
       use output
C
       implicit none
       integer                :: i,ii,j,jj,islap
       real(dp)               :: q,rd
       write(*,*)'Length of cubic edge, LCLUSTER = ',LCLUSTER,' a.u.'
       write(*,*)'r_s =',RS
       write(*,*)'Number of electrons, NE = ',NE
C  Open input and output data files
C  Contained in M_output.f
       call INITOUT
C  Read variables if not an input file is present 
C  Fill an array with a set of random numbers to be available;
C  Contained in M_random.f
       call INITRAN
C  Set the initial electron positions;
C  contained in M_random.f
       call INITRANNUMB
C  Associate electron with single particle wavefunction 
C  and determine initial wavefunction matrix;
C  Contained in M_orbital_cubicqd_cluster.f
       call INITORB
C  Set initial values for Jastrow parameters;
C  Contained in M_jastrow.f
       call INITJAS
C  Set initial values for inverse matrices;
C  Contained in M_determinant.f
       call INITDET
C  Set the initial values of the MC run;
C  Contained in M_observables.f
       call INITOBS
       write(*,*)'Maximum step interval= +-0.5*STEPMAX, STEPMAX = ',
     &  STEPMAX
       write(*,*)'Prerun and main run number of steps, MCPRE,MCMAX=',
     &                                                 MCPRE,MCMAX
C
C  MC loop:
C  First, the prerun for thermalizing
      lrun:do IMCR=1,MCMAX
       IMC = 0
C  Start the main run with statistical sampling of the observables if
C  the number of prerun steps is reached. Initialize the observables
C  and their statistics
       if ( IMCR == MCPRE) then
        write(*,*)'prerun: MCPRE= ',MCPRE,' acc. ratio = ',
     &           100.*DBLE(MCOUNT)/DBLE(NE*(MCPRE)),' %  '
        call INITRANOBS
       end if
       IMC=IMCR-MCPRE+1
C
C  Inner loop on electron index:
       lelrun:do IE=1,NE
C  Define some spin variables
        IES=1
        if (IE > NES(1)) IES=2
        SPINSEL=(IES-1)*NES(1)
        IEES=IE-SPINSEL
        IENS=NES(IES)
        do i=1,3
C  Shift position at random within +-STEPMAX/2
         rd = (FRAN(IFRAN)-0.5)*STEPMAX
         IFRAN = IFRAN + 1
         if (IFRAN > MRAN) then
          IRAN = IRAN + MRAN
          call FILLRAN
         end if
         RENEW(i,IE) = RE(i,IE)+rd
        end do
         call RDIST(RE,RENEW)
C  Calculate Jastrow factor quantities
        call JEXP
C  Calculate single particle wavefunction part
        call ORBWAV(RENEW(1:3,IE),PSINEW)
C  The matrix PSIMAT of the determinant gets its column IEES
        do ii=1,IENS
         PSIMAT(ii,IEES,IES)=PSINEW(SPINSEL+ii)
        end do
C   Calculate quotient QD, the ratio between new and old determinant,
C   as determinantal acceptance ratio
C   for the offered move of electron IEES of spin IES, calculated by
C   decomposition with respect to the cofactors
        QD(IES) =
     &   dot_product (AOLD(IEES,1:IENS,IES),PSIMAT(1:IENS,IEES,IES))
C  Test on acceptance
        q = (QD(IES)*QJ(IES))**2
        if ((dabs(RENEW(1,IE))/LCLUSTER >= 0.5) .or.
     &      (dabs(RENEW(2,IE))/LCLUSTER >= 0.5) .or.
     &      (dabs(RENEW(3,IE))/LCLUSTER >= 0.5)) then
         MCSTEP = .false.
        else 
         if (q < 1) then
          MCSTEP = (dble(FRAN(IFRAN)) < q)
          IFRAN = IFRAN + 1
          if (IFRAN .gt. MRAN) then
           IRAN=IRAN+MRAN
           call FILLRAN
          end if
         else
          MCSTEP = .true.
         end if
        end if
C        if (q < 1) then
C         MCSTEP = (dble(FRAN(IFRAN)) < q)
C         IFRAN = IFRAN + 1
C         if (IFRAN .gt. MRAN) then
C          IRAN=IRAN+MRAN
C          call FILLRAN
C         end if
C        else
C         MCSTEP = .true.
C        end if
C        if ((dabs(RENEW(1,IE))/LCLUSTER >= 0.5) .or.
C     &      (dabs(RENEW(2,IE))/LCLUSTER >= 0.5) .or.
C     &      (dabs(RENEW(3,IE))/LCLUSTER >= 0.5)) MCSTEP = .false.
C        if (MCSTEP .and. MCRUN) then
C        write(*,*)'main 2nd:IE,WAV = ',IE,PSINEW
C        write(*,*)'RENEW = ',RENEW(1:3,IE)
C        end if
C  Calculates observables
        if (MCRUN) then
         call OBSERV
        end if
C  Update for acceptance or not
        lmcstep:if (MCSTEP) then
         RE(1:3,IE) = RENEW(1:3,IE)
         call SLASM(QD(IES),AOLD(1:IENS,1:IENS,IES),
     &         PSIMAT(1:IENS,1:IENS,IES),ANEW(1:IENS,1:IENS,IES))
         AOLD(1:IENS,1:IENS,IES) = ANEW(1:IENS,1:IENS,IES)
         MCOUNT = MCOUNT + 1
        else lmcstep
         RENEW(1:3,IE) = RE(1:3,IE) ! forget the move
        end if lmcstep
C  Calculate statistical results of observables per electron
        if (MCRUN) call OBSERVSTATEL
       end do lelrun
C
C  Calculate statistical results of observables for all electrons
       if (MCRUN) call OBSERVSTATALL
      end do lrun
C
C  Do the output
C  Write density data on file
      call OUTWRITE
C  Write logfile with output data and close data files
      call OUTLOG
      stop
      end program cubic_cluster

C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCYCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
