C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      module random
       use highlevel
       use midlevel
C  controls the sequence of random numbers,
C  allows for use of different random generators
       implicit none
       integer,parameter,public :: MRAN=10001,ISEED=1820459103
       integer,public :: IRAN,IFRAN,ISEED0,ISEED1
       real,dimension(MRAN),public :: FRAN
       character(40), public :: RANDNAME
       public :: INITRAN,INITRANNUMB,FILLRAN,FILLG95,FILLTAO,RANF,
     &           AVVAR,BLOCKING
      contains
C-----------------------------------------------------------------------
      subroutine INITRAN
       MCPRE=100000
       MCMAX=2000000
       MAXA=100000
       STEPMAX=2.0_dp
       RANDNAME="random generator from TAO               "
       ISEED0 = ISEED
       CALL FILLRAN
      end subroutine INITRAN
C-----------------------------------------------------------------------
      subroutine INITRANNUMB
C  Set the initial electron positions at random
       integer   :: j,jj
       real(dp)  :: rd
       do j=1,NE
        do jj=1,3
         rd = 2.0_dp*(FRAN(IFRAN)-0.5_dp)*RCLUSTER
         RE(jj,j) = rd
         RENEW(jj,j) = rd
         IFRAN = IFRAN + 1
         if (IFRAN > MRAN) then
          IRAN = IRAN + MRAN
          call FILLRAN
         end if
        end do
       end do
      end subroutine INITRANNUMB
C-----------------------------------------------------------------------
       subroutine FILLRAN()
        select case (RANDNAME)
         case("random generator from g95               ")
           call FILLG95()
         case("random generator from TAO               ")
           call FILLTAO()
         case default
           call FILLG95()
        end select
       end subroutine FILLRAN
C
       subroutine FILLG95()
C        integer :: i
C        ISEED1 = ISEED
C        do i=1,MRAN
CC  below the g95 random generator which seems ok
C          FRAN(i) = rand(IRAN+i)
C        end do
C        IFRAN = 1
       end subroutine FILLG95
C-----------------------------------------------------------------------
       subroutine FILLTAO()
        integer :: i
        ISEED1 = ISEED0
        do i=1,MRAN
C  below from the book, "An Introduction to Computational Physics"
C  written by Tao Pang and published and copyrighted
C  by Cambridge University Press in 1997
          FRAN(i)  = RANF()
        end do
        ISEED0 = ISEED1
        IFRAN = 1
       end subroutine FILLTAO
C
C-----------------------------------------------------------------------
      function RANF() result(Z)
        real :: Z
        integer,parameter :: ia=16807,ic=2147483647,iq=127773,ir=2836
        integer :: ih,il,it
C  IC = 2**31-1 exists for 32 bit chips
        ih = ISEED1/iq
        il = MOD(ISEED1,iq)
        it = ia*il-ir*ih
        if (it.GT.0) then
          ISEED1 = it
        else
          ISEED1 = ic+it
        end if
        Z = ISEED1/FLOAT(ic)
      end function RANF
C-----------------------------------------------------------------------
      subroutine AVVAR(i,f,avgf,varf)
       integer,intent(in)       :: i
       real(dp),intent(in)      :: f
       real(dp),intent(inout)   :: avgf,varf
       if (i == 1) then
        avgf=f
        varf=0.0_dp
       else
        avgf=avgf+(f-avgf)/dble(i)
        varf=dble(i-2)/dble(i-1)*varf+
     &       dble(i)/dble(i-1)**2*(avgf-f)**2
       end if
      end subroutine AVVAR
C-----------------------------------------------------------------------
      subroutine BLOCKING(fa,avgfa,varfa,avgblocka,varblocka)
       real(dp),intent(in),dimension(NE)   :: fa
       real(dp),intent(inout),dimension(NE)  :: avgfa,varfa,avgblocka,
     &                                          varblocka
C  Blocks of length MAXA, where index IMCA counts within a block and
C  index IBLOCKA counts the MCMAX/MAXA blocks.
C  Input is the random variable fa and output is average and variance:
C  at each Monte-Carlo step avgfa,varfa and
C  after each block avblocka, varblocka.
C  The blocking is used for every electron separately.
       if (IBLOCKA > MCMAX/MAXA) write(*,*) 'Error BLOCKING:
     &    Step IMC=',IMC,' goes beyond last block IBLOCKA=',IBLOCKA,
     &    'which should be MCMAX/MAXA=',MCMAX/MAXA
       if (IMCA == 1) then
        avgfa(IE) = fa(IE)
        varfa(IE) = 0.0_dp
       else
        avgfa(IE)=avgfa(IE)+(fa(IE)-avgfa(IE))/dble(IMCA)
        varfa(IE)=dble(IMCA-2)/dble(IMCA-1)*varfa(IE)+
     &       dble(IMCA)/dble(IMCA-1)**2*(avgfa(IE)-fa(IE))**2
       end if
       if (IE == NE) then
        IMCA=IMCA+1
        if (IMCA == MAXA+1) then
C  Collect the blocking intermediate results
         if (IBLOCKA == 1) then
          avgblocka=avgfa
          varblocka=0.0_dp
         else
          avgblocka=avgblocka+
     &               (avgfa-avgblocka)/dble(IBLOCKA)
          varblocka=dble(IBLOCKA-2)/dble(IBLOCKA-1)*varblocka+
     &    dble(IBLOCKA)/dble(IBLOCKA-1)**2*(avgblocka-avgfa)**2
         end if
         IBLOCKA = IBLOCKA + 1
         IMCA = 1
        end if
       end if
      end subroutine BLOCKING
C-----------------------------------------------------------------------
      end module random
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
