CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCYCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      program Li_cluster
C  Calculates orbitals of Li atom
       use highlevel
       use midlevel
       use lattice_Lisolid
       use orbital_HF
       use determinant
       use random
       use jastrow, JEXP=>JASEXPSOLID
       use observables
       use output
       implicit none
       integer                :: i,ii,j,jj,islap,nslap1,nslap2,n1
       real(dp)               :: q,rd
       write(*,*)'Lattice constant, LATCON=',LATCON,'a.u.'
C  Open output data files
       call INITOUT
C  Fill an array with a set of random numbers to be available;
C  contained in M_random.f.
       MINOPT = 3.0_dp
       EGUESS = -2.490125_dp
       lslap1: do nslap1=10,10
       lslap2: do nslap2=1,2
C  RANDNAME = name of the random generator
C       RANDNAME="random generator from REC_PJN           "
       RANDNAME="random generator from TAO               "
C       RANDNAME="random generator from G95               "
C       RANDNAME="random generator from F90/95            "
       write(*,*)
       STEPMAX = 0.5_dp
       write(*,*)'Maximum step interval= +-0.5*STEPMAX, STEPMAX=',
     &  STEPMAX
       MCPRE = 100000
       MCMAX = 200000
       MAXA  = 100
       write(*,*)'Prerun and main run number of steps, MCPRE,MCMAX=',
     &                                                 MCPRE,MCMAX
       SLAPOPT(1)=0.900_dp
       SLAPOPT(2)=0.990_dp
       SLAP(1)=0.950_dp + (nslap1-10)*0.050_dp
       SLAP(2)=0.900_dp + (nslap2-10)*0.050_dp
       write(*,*)
       write(*,*)'Slater parameter loop, initial and increment,
     &            SLAP(1),SLAP(2)=',SLAP(1),SLAP(2)
       CJASOPT = 1.00_dp
       ljas: do n1=1,2
       CJAS=0.000000001_dp+1.00_dp+(n1-10)*0.10_dp
       write(*,*)'SLAP(1),SLAP(2),CJAS=',SLAP(1),SLAP(2),CJAS
       write(*,*)
       call INITRAN
C  Set the lattice
       call INITLATTICE
C  Set the initial electron positions;
C  contained in M_random.f
CC Has been done in INITLATTICE
C       call INITRANNUMB
C  Associate electron with single particle wavefunction, normalize
C  wavefunction, and determine initial wavefunction matrix;
C  contained in M_orbital_Li.f
       call INITORB
       call INITORB1
C  Set initial values for Jastrow parameters
       call INITJAS
C  Set initial values for inverse matrices;
C  contained in M_determinant.f
       call INITDET
C  Set the initial values of the MC run;
C  contained in M_observables.f
       call INITOBS
       WPSI = 1.0_dp
       SWEIGHT = 0.0_dp
C
C  MC loop:
      MCOUNT = 0
C  First, the prerun for thermalizing
      lrun:do IMCR=1,MCMAX
       IMC = 0
C  Start the main run with statistical sampling of the observables if
C  the number of prerun steps is reached. Initialize the observables
C  and their statistics
       if ( IMCR == MCPRE) then
        write(*,*)'prerun: MCPRE= ',MCPRE,' acc. ratio = ',
     &           100.*DBLE(MCOUNT)/DBLE(NE*(MCPRE)),' %  '
        call INITRANOBS
       end if
       IMC=IMCR-MCPRE+1
C
C  Inner loop on electron index:
       lelrun:do IE=1,NE
        IES=1
        IEES=IE
        if (IE > NES(1)) then
         IES=2
         IEES=IE-NES(1)
        end if
        SPINSEL=(IES-1)*NES(1)
        IENS=NES(IES)
        if (IENS /= 3*NK/2) then
         write(*,*) 'astarra: compare',IENS-NK,'=',NK/2
         stop
        end if
        do i=1,3
C  Shift position at random within +-STEPMAX/2
         rd = (FRAN(IFRAN)-0.5)*STEPMAX
         IFRAN = IFRAN + 1
         if (IFRAN > MRAN) then
          IRAN = IRAN + MRAN
          call FILLRAN
         end if
         RENEW(i,IE) = RE(i,IE)+rd
        end do
        ZENEW(1:3,IE) = RENEW(1:3,IE)/SULATCON
C  Determine the new electron-nuclei distance vectors
        call  RDISTELNU(RENEW(1:3,IE),RK(1:3,1:NK),RKENEW(1:4,1:NK,IE))
        ZKENEW(1:3,1:NK,IE) = RKENEW(1:3,1:NK,IE)/LATCON
C  Calculate Jastrow factor quantities
        call JEXP
C  Calculate single particle wavefunction part
          call ORBWAV(RKENEW(4,1:NK,IE),WPHI)
C  Distribute 3*NK electrons upon orbitals of NK nuclei:
C  1st orbital on all NK nuclei for both spins,
C  2nd orbital on first nuclei for IES=1 and on last NK/2 nuclei for
C  spin IES=2.
        do ii=1,NK
         PSINEW(ii) = WPHI(ii,NELORB(SPINSEL+ii))
        end do
C           call LIDIMER(WPHI,WHYPHI) !for hybrid states
        do ii=NK+1,3*NK/2
C         PSINEW(ii) = WHYPHI(ii-NK+NK/2*(IES-1),NELORB(ii)) !for hybrid states
         PSINEW(ii) = WPHI(ii-NK+NK/2*(IES-1),NELORB(SPINSEL+ii))
        end do
C  The matrix PSIMAT of the determinant gets its column IEES
        do ii=1,IENS
         PSIMAT(ii,IEES,IES)=PSINEW(ii)
        end do
C   Calculate quotient QD, the ratio between new and old determinant,
C   as determinantal acceptance ratio
C   for the offered move of electron IEES of spin IES, calculated by
C   decomposition with respect to the cofactors
        QD(IES) =
     &   dot_product (AOLD(IEES,1:IENS,IES),PSIMAT(1:IENS,IEES,IES))
C  Test on acceptance
        q = (QD(IES)*QJ(IES))**2
        if (q < 1) then
         MCSTEP = (dble(FRAN(IFRAN)) < q)
         IFRAN = IFRAN + 1
         if (IFRAN .gt. MRAN) then
          IRAN=IRAN+MRAN
          call FILLRAN
         end if
        else
         MCSTEP = .true.
        end if
C  Calculates observables
        if (MCRUN) then
         call OBSERV
        end if
C  Update for acceptance or not
        lmcstep:if (MCSTEP) then
         RKE(1:4,1:NK,IE) = RKENEW(1:4,1:NK,IE)
         RE(1:3,IE) = RENEW(1:3,IE)
         call SLASM(QD(IES),AOLD(1:IENS,1:IENS,IES),
     &         PSIMAT(1:IENS,1:IENS,IES),ANEW(1:IENS,1:IENS,IES))
         AOLD(1:IENS,1:IENS,IES) = ANEW(1:IENS,1:IENS,IES)
         IKMOLD(IE) = IKM(IE)
         MCOUNT = MCOUNT + 1
        else lmcstep
         RENEW(1:3,IE) = RE(1:3,IE) ! forget the move
         RKENEW(1:4,1:NK,IE) = RKE(1:4,1:NK,IE)
        end if lmcstep
C  Calculate statistical results of observables per electron
        if (MCRUN) call OBSERVSTATEL
       end do lelrun
C
C  Calculate statistical results of observables for all electrons
       if (MCRUN) call OBSERVSTATALL
      end do lrun
C
C  Do the output
C  Write density data on file
      call OUTWRITE
C  Write logfile with output data and close data files
      call OUTLOG
      end do ljas
      end do lslap2
      end do lslap1
      close(31)
      stop
      end program Li_cluster

C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCYCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
