C-----------------------------------------------------------------------
      module lattice_Lisolid
       use highlevel
       use midlevel
       use random
C
       implicit none
       integer,parameter,public                      :: GMAX=10
       integer,public                                :: GLASTR,GLASTI
       integer,dimension(N1D**4),public              :: SQG
       real(dp),dimension(3,N1D**3),public           :: SZK
       real(dp),dimension(3,N1D**4),public           :: SGK
       integer,dimension(N1d**4)                     :: GCUTR,GCUTI
       real(dp),dimension(3,(2*GMAX+1)**3),public    :: GK
       real(dp),dimension(N1D**4,NK),public          :: RLCAO,ILCAO
       character(len=14),parameter,public  :: INNAME = 'LATTICE_LI.DAT'
       public  :: INITLATTICE,MAKE_SUPERLATTICE,MAKE_LATTICE
      contains
C-----------------------------------------------------------------------
      subroutine INITLATTICE
C
      integer  :: i,ij,ik,ike,mi,hik,hlr,hli,h2
      integer,dimension(N1D**4)  :: hcutr,hcuti
      real(dp) :: h1
      open(40,file=INNAME,status='unknown')
      call MAKE_SUPERLATTICE(1,SZK,SGK,mi,SQG)
      if (mi /= N1D**3) then
       write (*,*)'N1D**3 not equal sc lattice points mi = ',mi
       stop
      end if
C  Cell coordinates of nuclei in units
C  of the primitive cube containing 2 bcc lattice cells
      call MAKE_LATTICE(3,N1D,ZK,GK,GMAX,mi)
      if (mi /= 2*N1D**3) then
       write (*,*)'2*N1D**3 not equal number of nuclei = ',mi
       stop
      end if
C  Transform nuclei coordinates to Bohr (a.u.)
      RK(1:3,1:NK) = ZK(1:3,1:NK)*LATCON
C  Electron positions in Bohr
      RANDNAME="random generator from TAO               "
      i=0
      call INITRAN
      call INITRANNUMB
      do ij=1,NE
       ike=mod(ij,NK)
       RE(1:3,ij) = RENEW(1:3,ij) + RK(1:3,ike) + 0.5_dp
      end do
C
C  Fold back into the Wigner-Seitz sc cell of the superlattice
C  Electrons:
       ZE(1:3,1:NE) = RE(1:3,1:NE)/SULATCON
C  Put ZE into [-1/2,1/2]-cube, i.e. fold back
       do ij=1,NE
        call FOLDTOWSC(ZE(1:3,ij))
       end do
       RE(1:3,1:NE) = ZE(1:3,1:NE)*SULATCON
C  Put into cell coordinates of physical unit cell
       ZE(1:3,1:NE) = RE(1:3,1:NE)/LATCON
C  Nuclei (analogous to electrons):
       ZK(1:3,1:NK) = RK(1:3,1:NK)/SULATCON
       do ik=1,NK
        call FOLDTOWSC(ZK(1:3,ik))
       end do
       RK(1:3,1:NK) = ZK(1:3,1:NK)*SULATCON
       ZK(1:3,1:NK) = RK(1:3,1:NK)/LATCON
C  LCAO coefficients exp(iZK(ij)*SGK(ik)), ik=(1:NK/2),ij=(1:NK)
C  Use "real" and "im" part: RLCAO and ILCAO
       write (40,*)'lattice:'
       write (40,'(3(2x,i3,3f7.3))') (ij,ZK(1:3,ij),ij=1,NK)
       do ik=1,NK/2
        do ij=1,NK
         h1 = dot_product (SGK(1:3,ik),ZK(1:3,ij))
         RLCAO(ik,ij) = dcos (h1)
         ILCAO(ik,ij) = dsin (h1)
        end do
       end do
       hcutr=1
       hcuti=1
       hlr=0
       hli=0
       h2=0
       hcuti(1)=0     ! zero because rec.lat.point has no imag.part
       do ik=1,NK/2
        if (hcutr(ik) == 0) cycle
        h2=h2+1
        if (h2 == NK/4+1)then
         GLASTR=ik
         hlr=h2
C         write (*,*) 'hlr,GLASTR=',hlr,GLASTR
        end if
        do hik=1,NK/2
         if (hik == ik) cycle
         if ((sum (dabs (RLCAO(ik,1:NK)-RLCAO(hik,1:NK))) < EMACH)
     &    .or. (sum (dabs (RLCAO(ik,1:NK)+RLCAO(hik,1:NK))) < EMACH))
     &      hcutr(hik)=0
        end do
       end do
       h2=0
       do ik=1,NK/2
        if (hcuti(ik) == 0) cycle
        h2=h2+1
        if ((hlr+h2) == NK/2) then
         GLASTI=ik
         hli=h2
C         write (*,*)'hli,GLASTI=',hli,GLASTI
        end if
        do hik=1,NK/2
         if (hik == ik) cycle
         if ((sum (dabs (ILCAO(ik,1:NK)-ILCAO(hik,1:NK))) < EMACH)
     &    .or. (sum (dabs (ILCAO(ik,1:NK)+ILCAO(hik,1:NK))) < EMACH))
     &      hcuti(hik)=0
        end do
       end do
       write (40,*)
       write (40,*)'cut the following states hcutr=0 from real part:'
       write (40,*)'ik  hcutr SGK(1) SGK(2) SGK(3)  SQG'
       write (40,'(64(i3,x,i3,2x,3f7.3,x,i3/))')
     &    (ik,hcutr(ik),SGK(1:3,ik)*N1D/2/PI,SQG(ik),ik=1,NK/2)
       write (40,*)'cut the following states hcuti=0 from imag. part:'
       write (40,*)'ik  hcuti SGK(1) SGK(2) SGK(3)  SQG'
       write (40,'(64(i3,x,i3,2x,3f7.3,x,i3/))')
     &    (ik,hcuti(ik),SGK(1:3,ik)*N1D/2/PI,SQG(ik),ik=1,NK/2)
      GCUTR=hcutr
      GCUTI=hcuti
      write (*,*)'max. rec.lat.vectors for LCAO wave:'
      write (*,*)'real part: # and last index      =',hlr,GLASTR
      write (*,*)'imaginary part: # and last index =',hli,GLASTI
C  Calculate the distances
C  Between electrons and nuclei:
       do i=1,NE
        call RDISTELNU(RE(1:3,i),RK(1:3,1:NK),RKE(1:4,1:NK,i))
        ZKE(1:3,1:NK,i) = RKE(1:3,1:NK,i) / LATCON
       end do
C  Between electrons and electrons:
       call RDISTELEL(RE,RE,REE,ZEE)
C  Cut-off from zero is introduced by JASEMACH
        REE(4,1:NE,1:NE) = max (REE(4,1:NE,1:NE),JASEMACH)
       close(40)
       end subroutine INITLATTICE
C-----------------------------------------------------------------------
       subroutine MAKE_SUPERLATTICE(la,r,g,mi,sqg)
C  Construct the superlattice and its reciprocal lattice.
C  They are both always simple cubic. The
C  lengths have to be multiplied by LATCON to go from cell coordinates
C  to atomic units.
C  la lattice type
C  r lattice vector
C  g reciprocal lattice vector
C  dg degeneracy of rec.lat.vector
C  sqg square of rec.lat.vector
C  mi number of lattice vectors r
        integer,intent(in) :: la
        integer,intent(out):: mi
        integer,intent(out),dimension(N1D**4)    :: sqg
        real(dp),intent(out),dimension(3,N1D**3) :: r
        real(dp),intent(out),dimension(3,N1D**4) :: g
        integer :: i1,i2,i3,m1,m2,ind,gind,g2,g1
        integer,dimension(N1D**4) :: dg
        if (la /= 1) then
         write (*,*)'error: superlattice is not simple cubic, la = ',la
         stop
        end if
        if (N1D/2*2 == N1D) then
         m1=-N1D/2+1
         m2=-m1+1
        else
         m1=-(N1D-1)/2
         m2=-m1
        end if
        ind = 0
        do i1=m1,m2
         do i2=m1,m2
          do i3=m1,m2
           ind=ind+1
           r(1,ind)=i1
           r(2,ind)=i2
           r(3,ind)=i3
          end do
         end do
        end do
        mi =ind
        if (mi /= N1D**3) then
         write(*,*) 'MAKE_SUPERLATTICE Hirsch:',
     &    ' cannot count!',N1D**3,'=',mi,'!'
        end if
        sqg=0
        gind=0
        dg=0
        lrec:do g2=0,N1D**4
        g1=0
        do i1=-N1D,N1D
         do i2=-N1D,N1D
          do i3=-N1D,N1D
           if (gind+1 > N1D**4) exit lrec
           if(i1**2+i2**2+i3**2 == g2) then
            gind=gind+1
            g(1,gind)=i1
            g(2,gind)=i2
            g(3,gind)=i3
            sqg(gind)=g2
            g1=g1+1
           end if
          end do
         end do
        end do
        dg(g2)=g1
        end do lrec
        write (40,*)'reciprocal lattice: i,g(1:3,i)*N1D/(2*PI),sqg(i):'
        write (40,*)'according to degeneracy some more states are listed
     &  after the final state NK/2'
        do i1=1,NK/2+dg(sqg(NK/2+1))
         if (sqg(i1) > sqg(NK/2)) exit
         write (40,'(x,i4,"  ("3i2")",i4)') i1,int(g(1:3,i1)),sqg(i1)
        end do
        g=2._dp*PI*g/N1D
       end subroutine MAKE_SUPERLATTICE
C-----------------------------------------------------------------------
       subroutine MAKE_LATTICE(la,mr,r,g,gm,mi)
C   Calculates the cubic cartesian coordinates for the lattice points
C   of simple cubic la=SC=1 and body-centered la=BCC=3 lattice
C   r(1:3,ind) = three cartesian coordinates for a lattice point "ind"
C   in a cubic mesh of size (-mr:mr, -mr:mr, -mr:mr).
C   The reciprocal lattice points g(1:3,gind) are calculated for
C   (2*gm+1)**3 for bcc and with same number as direct lattice
C   for sc.
        integer,intent(in)   :: la,mr,gm
        integer,intent(out)  :: mi
        real(dp),intent(out),dimension(3,2*mr**3) :: r
        real(dp),intent(out),dimension(3,(2*gm+1)**3) :: g
        integer i1,i2,i3,ind,m1,m2,gind
        if (mr/2*2 == mr) then
         m1=-mr/2+1
         m2=-m1+1
        else
         m1=-(mr-1)/2
         m2=-m1
        end if
        if (la == 1) then
         ind=0
         gind=0
         do i1=m1,m2
          do i2=m1,m2
           do i3=m1,m2
            ind=ind+1
            gind=gind+1
            r(1,ind)=i1
            r(2,ind)=i2
            r(3,ind)=i3
            g(1,gind)=2._dp*PI*i1
            g(2,gind)=2._dp*PI*i2
            g(3,gind)=2._dp*PI*i3
           end do
          end do
         end do
        else if (la == 3) then
         ind=0
         do i1=m1,m2
          do i2=m1,m2
           do i3=m1,m2
            ind=ind+1
            r(1,ind)=+i1
            r(2,ind)=+i2
            r(3,ind)=+i3
           end do
          end do
         end do
         do i1=m1,m2
          do i2=m1,m2
           do i3=m1,m2
            ind=ind+1
            r(1,ind)=0.5_dp+i1
            r(2,ind)=0.5_dp+i2
            r(3,ind)=0.5_dp+i3
           end do
          end do
         end do
         gind=0
         do i1=-gm,gm
          do i2=-gm,gm
           do i3=-gm,gm
            gind=gind+1
            g(1,gind)=2._dp*PI*(i2+i3)
            g(2,gind)=2._dp*PI*(i1+i3)
            g(3,gind)=2._dp*PI*(i1+i2)
           end do
          end do
         end do
         mi=ind
         if (ind /= 2*mr**3) then
          write(*,*) 'MAKE_LATTICE Hirsch:',
     &       'cannot count!',2*mr**3,'=',ind
         end if
        else
         write(*,*) "nothing for that lattice, la=",la
         stop
        end if
       end subroutine MAKE_LATTICE
C-----------------------------------------------------------------------
      end module lattice_Lisolid
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
