C-----------------------------------------------------------------------
      module lattice_Lisolid
       use highlevel
       use midlevel
C
       implicit none
       character(len=14),parameter,public  :: INNAME = 'LATTICE_LI.DAT'
       public  :: INITLATTICE,MAKE_LATTICE
      contains
C-----------------------------------------------------------------------
      subroutine INITLATTICE
C
      integer  :: i,ij,ik,ike
C  Load lattice from file
       open (32,FILE=INNAME,STATUS='OLD')
C  Read the nuclei positions
       do ik=1,NK
C  Cell coordinates of nuclei are read, i.e. in units
C  of the primitive cube containing 2 bcc lattice cells
        read (32,*) ZK(1,ik),ZK(2,ik),ZK(3,ik)
       end do
C  Transform nuclei coordinates to Bohr
       RK(1:3,1:NK) = ZK(1:3,1:NK)*LATCON
C  Read electron positions in Bohr
       do ij=1,NE
        read (32,*) RE(1,ij),RE(2,ij),RE(3,ij),ike
        RE(1:3,ij) = RE(1:3,ij) + RK(1:3,ike)
       end do
       close (32)
C
C  Fold back into the Wigner-Seitz sc cell of the superlattice
C  Electrons:
       ZE(1:3,1:NE) = RE(1:3,1:NE)/SULATCON
C  Put ZE into [-1/2,1/2]-cube, i.e. fold back
        do ij=1,NE
         call FOLDTOWSC(ZE(1:3,ij))
        end do
       RE(1:3,1:NE) = ZE(1:3,1:NE)*SULATCON
C  Put into cell coordinates of physical unit cell which is dominantly
C  used in the program
       ZE(1:3,1:NE) = RE(1:3,1:NE)/LATCON
C  Nuclei (analogous to electrons):
        ZK(1:3,1:NK) = RK(1:3,1:NK)/SULATCON
        do ik=1,NK
         call FOLDTOWSC(ZK(1:3,ik))
        end do
        RK(1:3,1:NK) = ZK(1:3,1:NK)*SULATCON
        ZK(1:3,1:NK) = RK(1:3,1:NK)/LATCON
C        write(33,*)'RK(1:3,1:NK)=',(ij,RK(1:3,ij),ij=1,NK)
C        write(33,*)'RE(1:3,1:NE)=',(ij,RE(1:3,ij),ij=1,NE)
C
C  Calculate the distances
C  Between electrons and nuclei:
        do i=1,NE
         call RDISTELNU(RE(1:3,i),RK(1:3,1:NK),RKE(1:4,1:NK,i))
         ZKE(1:3,1:NK,i) = RKE(1:3,1:NK,i) / LATCON
         call MINDIST(NK,RKE(1:4,1:NK,i),IKM(i))
        end do
C  Between electrons and electrons:
        call RDISTELEL(RE,RE,REE,ZEE)
C  !!!a cut-off from zero is introduced by JASEMACH!!!
        REE(4,1:NE,1:NE) = max (REE(4,1:NE,1:NE),JASEMACH)
       end subroutine INITLATTICE
C-----------------------------------------------------------------------
       subroutine MAKE_LATTICE(la,mr,r)
        integer,intent(in) :: la,mr
        real(dp),intent(out),dimension(3,mr) :: r
        integer i1,i2,i3,ind
        ind = 0
        if (la == 1) then
         do i1=-mr,mr
          do i2=-mr,mr
           do i3=-mr,mr
            ind=ind+1
            r(1,ind)=i1
            r(2,ind)=i2
            r(3,ind)=i3
           end do
          end do
         end do
        else if (la == 3) then
         do i1=-mr,mr
          do i2=-mr,mr
           do i3=-mr,mr
            ind=ind+1
            r(1,ind)=i1+i2-i3
            r(2,ind)=-i1+i2+i3
            r(3,ind)=i1+i2+i3
           end do
          end do
         end do
        else
         write(*,*) "nothing for that lattice, la=",la
         stop
        end if
        if (ind /= (2*mr+1)**3) then
         write(*,*) 'pello: cannot count!',(2*mr+1)**3,'=',ind
        end if
       end subroutine MAKE_LATTICE
C-----------------------------------------------------------------------
      end module lattice_Lisolid
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
