C-----------------------------------------------------------------------
      module observables
       use highlevel
       use midlevel
       use random
       use orbital_hf
       use determinant
       use jastrow
       use lattice_Lisolid
       use ewald_cal_tab
C Comprises subroutines for potential, interaction and kinetic energy
C  INTEN  interaction energy (INTENOLD = old value before new draw), a
C         specific electron is considered
C  POTEN  potential energy (in the following OLD appended as above)
C  VIREN  addition for virial theorem
C  KINEN  kinetic energy; kinetic energy is also a two-body expression
C         and has to be updated at each step
C  TOTELEN sum of kinetic,potential and interaction energy
C          for specific electron
C  NUNUEN static nucleus-nucleus energy for the whole primitive
C         superlattice cell, contains also constants from other energies
C  MPC  model-periodic Coulomb potential denisty correction
C  AVELEN  average over full QMC run of TOTELEN
C  VARELEN variance of TOTELEN
C  AVALLEL average of TOTELEN over NE electrons, i.e.total
C         energy of all electrons per electron
C  VARALLEL variance of total energy per electron
C  AVBLEL average of total energy as above, intermediately used only
C         for input and control of blocking
C  VARBLEL variance of total energy per electron
C  AVKINEL average of single electron laplace kinetic energy
C  VARKINEL variance of AVKINEL
C  AVVELEL average of single electron velocity kinetic energy
C  VARVELEL variance of AVVELEL
C  AVPOTEL average of single electron potential energy
C  VARPOTEL variance of AVPOTEL
C  AVINTEL average of single electron part of interaction energy
C  VARINTEL variance of AVINTEL
C  AVTOTKIN average of all electrons' kinetic energy per electron
C  VARTOTKIN variance of AVTOTKIN, similarly for the other
C           energy parts, i.e. AVTOTPOT,AVTOTINT
C  AVBLOCKEL block average of single electron total energy
C  VARBLOCKEL block variance of AVBLOCKEL
C  AVALLBLOCKS block average over all electrons
C  VARALLBLOCKS block variance over all electrons
C  RHORAD radial density with respect to nearest nucleus
C  LOCDEN local density in superlattice sc Wigner-Seitz cell
C  DENINT integrated density = number of electrons!
       implicit none
       integer,parameter,public        :: NRHO=1000
       real(dp),public   :: AVTOTALL,AVALLEL,VARTOTALL,DRHO,NUNUEN,
     &       MPC,ALLKIN,ALLPOT,ALLVIR,ALLINT,AVTOTKIN,VARTOTKIN,
     &       AVTOTPOT,VARTOTPOT,AVTOTVIR,VARTOTVIR,AVTOTINT,VARTOTINT,
     &       ALLBLOCKS,AVALLBLOCKS,VARALLBLOCKS,DENINT
       real(dp),public,dimension(NE)   :: INTEN,POTEN,KINEN,
     &       POTENOLD,VIREN,VIRENOLD,KINENOLD,VELEN,VELENOLD,TOTELEN,
     &       AVELEN,VARELEN,AVBLEL,VARBLEL,AVKINEL,VARKINEL,AVVELEL,
     &       VARVELEL,AVPOTEL,VARPOTEL,AVVIREL,VARVIREL,AVINTEL,
     &       VARINTEL,AVBLOCKEL,VARBLOCKEL
       real(dp),public,dimension(NRHO,NE) :: RHORAD,RHORADOLD,AVRHORAD
       real(dp),public,dimension(NDEN,NDEN,NDEN) :: AVLOCDEN
       public :: INITOBS,OBSERV,OBSERVSTATEL,OBSERVSTATALL
       public :: INTENERGY,POTENERGY,KINENERGY,DENSITY
       public :: QUADRATURE_SCWSC,LOCAL_DENSITY
      contains
C-----------------------------------------------------------------------
      subroutine INITOBS
       integer               :: i,ii
       real(dp),dimension(4) :: h
       MCRUN=.false.
       INTEN=EMACH
       POTEN=EMACH
       VIREN=EMACH
       KINEN=EMACH
       VELEN=EMACH
       TOTELEN=EMACH
       DRHO=0.01
       RHORAD=0.0_dp
C  Calculate the nuclei lattice energy per electron.
       NUNUEN = EMACH
       write(*,*)'start of nuclear energy calculation'
       do ii=1,NK
        do i=1,NK
         h(1:3)=(ZK(1:3,i)-ZK(1:3,ii))/dble(N1D)
         call FOLDTOWSC(h(1:3))
         h(4) = dsqrt (sum (h(1:3)**2))
         if (h(4) == 0._dp) cycle
C         call EWALD(h(1:3),VEW_CAL,VEWGR_CAL)
         VEW_CAL = VEW(SULATTY,h(1:3))
C         write(*,'(1x,4f7.3,1f9.5)')h,VEW_CAL
         NUNUEN = NUNUEN + VEW_CAL
        end do
       end do
       NUNUEN = +0.5_dp*CHARGE*(NUNUEN/NK + VMAD)/SULATCON
C  Madelung term per superlattice unit cell and per electron is
C  included here.
       write(*,*)'nuclear energy NUNUEN=',NUNUEN,'includes Madelung
     &  constant VMAD for zero potential'
C       stop
      end subroutine INITOBS
C-----------------------------------------------------------------------
      subroutine INITRANOBS
       MCRUN = .true.
       MCOUNT = 0
       IBLOCKA = 1
       IMCA = 1
       AVELEN=0.0_dp
       AVALLEL=0.0_dp
       VARELEN=0.0_dp
       AVTOTALL=0.0_dp
       VARTOTALL=0.0_dp
       AVKINEL=0.0_dp
       VARKINEL=0.0_dp
       AVBLEL=0.0_dp
       VARBLEL=0.0_dp
       AVBLOCKEL=0.0_dp
       VARBLOCKEL=0.0_dp
       AVALLBLOCKS=0.0_dp
       VARALLBLOCKS=0.0_dp
       AVRHORAD=0.0_dp
       AVLOCDEN=0.0_dp
      end subroutine INITRANOBS
C-----------------------------------------------------------------------
      subroutine OBSERV
C  The actual random values of the observables are called if the step
C  is accepted, otherwise the old values are taken except the
C  interaction energy which could have changed because positions
C  of other electrons have changed after the last access to this
C  electron.
       call INTENERGY(INTEN)
       call KINENERGY(KINEN,VELEN)
       call LOCAL_DENSITY
       if(MCSTEP) then
        call POTENERGY(POTEN,VIREN)
        call DENSITY
       else
        POTEN(IE)=POTENOLD(IE)
        VIREN(IE)=VIRENOLD(IE)
        RHORAD(1:NRHO,IE)=RHORADOLD(1:NRHO,IE)
       end if
       TOTELEN(IE)=KINEN(IE)+POTEN(IE)+INTEN(IE)
      end subroutine OBSERV
C-----------------------------------------------------------------------
      subroutine OBSERVSTATEL
C  The statistics for the observables is calculated for each electron
C  separately. The new values are named as old ones for the start of a
C  new IMC step.
         POTENOLD(IE)=POTEN(IE)
         VIRENOLD(IE)=VIREN(IE)
         RHORADOLD(1:NRHO,IE)=RHORAD(1:NRHO,IE)
         call AVVAR(IMC,TOTELEN(IE),AVELEN(IE),VARELEN(IE))
         call AVVAR(IMC,KINEN(IE),AVKINEL(IE),VARKINEL(IE))
         call AVVAR(IMC,VELEN(IE),AVVELEL(IE),VARVELEL(IE))
         call AVVAR(IMC,POTEN(IE),AVPOTEL(IE),VARPOTEL(IE))
         call AVVAR(IMC,VIREN(IE),AVVIREL(IE),VARVIREL(IE))
         call AVVAR(IMC,INTEN(IE),AVINTEL(IE),VARINTEL(IE))
         AVRHORAD(1:NRHO,IE)=AVRHORAD(1:NRHO,IE)*dble(IMC)/dble(IMC+1)+
     &              RHORAD(1:NRHO,IE)/dble(IMC+1)
         call BLOCKING(TOTELEN,AVBLEL,VARBLEL,AVBLOCKEL,VARBLOCKEL)
      end subroutine OBSERVSTATEL
C-----------------------------------------------------------------------
      subroutine OBSERVSTATALL
C  Calculates the overall per electron statistics, i.e. for the
C  average of NE electrons.
       AVALLEL = sum (TOTELEN(1:NE))/dble(NE)
       call AVVAR(IMC,AVALLEL,AVTOTALL,VARTOTALL)
       ALLKIN = sum (KINEN(1:NE))/dble(NE)
       call AVVAR(IMC,ALLKIN,AVTOTKIN,VARTOTKIN)
       ALLPOT = sum (POTEN(1:NE))/dble(NE)
       call AVVAR(IMC,ALLPOT,AVTOTPOT,VARTOTPOT)
       ALLVIR = sum (VIREN(1:NE))/dble(NE)
       call AVVAR(IMC,ALLVIR,AVTOTVIR,VARTOTVIR)
       ALLINT = sum (INTEN(1:NE))/dble(NE)
       call AVVAR(IMC,ALLINT,AVTOTINT,VARTOTINT)
       ALLBLOCKS = sum (AVBLOCKEL(1:NE))/dble(NE)
       call AVVAR(IMC,ALLBLOCKS,AVALLBLOCKS,VARALLBLOCKS)
      end subroutine OBSERVSTATALL
C-----------------------------------------------------------------------
      subroutine INTENERGY(inten)
C  Calculates interaction energy. The factor 0.5 at the Coulomb double
C  sum is taken into account here, inten(IE)=0.5*\sum_k 1/|r_IE - r_k| +
C  Observe that coordinates within the Wigner-Seitz cell
C  are used in units of SULATCON. The density-density correction of
C  the model-periodic-Coulomb energy for electrons is computed
C  with the densities at the end of the run.
C
C  inten(IE)= contribution to the interaction energy of IE-th electron
       real(dp),intent(out),dimension(NE) :: inten
C
       integer :: i,k
       real(dp) :: hw
       real(dp),dimension(4) ::hrj
       hw=0.0_dp
C  Coulomb el-el interaction is summed
       ielekpot:do k=1,NE
        if (k == IE) then
         cycle ielekpot
        end if
        if (MCSTEP) then
         hrj(1:4) = DISTNEW(1:4,IE,k)/SULATCON
        else
         hrj(1:4) = DIST(1:4,IE,k)/SULATCON
        end if
        call FOLDTOWSC(hrj(1:3))
        hrj(4) = dsqrt( sum (hrj(1:3)**2))
        hrj(4) = max (hrj(4),EMACH)
        hw = hw + 1.0_dp/hrj(4)
       end do ielekpot
       inten(IE)=0.5_dp*hw/SULATCON
      end subroutine INTENERGY
C-----------------------------------------------------------------------
      subroutine POTENERGY(poten,viren)
C  Calculates one-particle potential energy.
C  The energy is composed of the centrosymmetric potentials of all
C  nuclei via Ewald.
C  The additional term arising in the virial theorem,
C  E_{vir} = -3\sum_{i,n} R_n(r_i-R_n)/|r_i-R_n|^3 = VIREN,
C  is calculated. When calculating without Ewald correction the theorem
C  reads as: 2E_{kin}+E_{int}+E_{pot}+E_{vir} = 0
       real(dp),intent(out),dimension(NE)  :: poten,viren
C
       integer  ::  ik
       real(dp)  ::  hp,hvir,hv
       real(dp),dimension(3,NK) :: er,ez
       real(dp),dimension(4)    :: r
       hp = 0.0_dp
       hvir =0.0_dp
       do ik =1,NK
        r(1:4) = RKENEW(1:4,ik,IE)/SULATCON
        r(4) = max (r(4),JASEMACH)
        hv=dot_product (r(1:3),RK(1:3,ik)) /r(4)**3
C        call EWALD(r(1:3),VEW_CAL,VEWGR_CAL)
        VEW_CAL = VEW(SULATTY,r(1:3))
        hp = hp + VEW_CAL
        hvir = hvir +  hv
       end do
       viren(IE) = hvir*CHARGE/SULATCON**2
       poten(IE) = -hp*CHARGE/SULATCON
C----------end new lattices' division-----------------------------------
      end subroutine POTENERGY
C-----------------------------------------------------------------------
      subroutine KINENERGY(kinen,velen)
C  Calculates kinetic energy kinen(IE) of electron IE.
C  It includes the kinetic energy originating from the determinant
C  and the Jastrow factor. The latter is a two-body term which
C  is completed by the Ewald potential to reflect the periodic system
C  completion. Here, in the Jastrow, factor we use the smoothly
C  continuous potential to get rid of artificial fluctuations. We are
C  allowed to do so because it is just an Ansatz for optimizing
C  the wavefunction.
       real(dp),intent(out),dimension(NE)    :: kinen,velen
C
       integer                               :: iii
       real(dp)                              :: hkin,qsel
       real(dp)                              :: hlad,hlaj
       real(dp),dimension(3)                 :: hgrd,hgrj
       real(dp),dimension(3,NK,NORB)         :: pgrnew
       real(dp),dimension(NK,NORB)           :: planew
       if (MCSTEP) then
        qsel = QD(IES)
        select case (ORBNAME)
         case("Compute Slater orbitals                 ")
          call ORBDER(RKENEW(1:4,1:NK,IE),pgrnew,planew)
         case("Read Slater orbitals from spline file   ")
          call ORBDERSPLINE(RKENEW(1:4,1:NK,IE),HRHS(IE,1:NK),
     &                    HIHS(IE,1:NK),pgrnew,planew)
         case default
          stop
        end select
        hgrj(1:3)=GRJAS(1:3,IEES,IES)
        hlaj=LAPJAS(IEES,IES)
       else
        qsel = 1._dp
        select case (ORBNAME)
         case("Compute Slater orbitals                 ")
          call ORBDER(RKE(1:4,1:NK,IE),pgrnew,planew)
         case("Read Slater orbitals from spline file   ")
          call ORBDERSPLINE(RKE(1:4,1:NK,IE),HRHSOLD(IE,1:NK),
     &                    HIHSOLD(IE,1:NK),pgrnew,planew)
         case default
          stop
        end select
        hgrj(1:3)=GRJASOLD(1:3,IEES,IES)
        hlaj=LAPJASOLD(IEES,IES)
       end if
C       write(34,*)'IE,IES=',IE,IES
C       write(34,*)'planew(1:NK,1:NORB),HIHSOLD(IE,iii),HIHS(IE;iii)='
C       do iii=1,NK
C        write(34,*)'iii=',iii,planew(iii,1:NORB),HIHSOLD(IE,iii),
C     &    HIHS(IE,iii)
C       end do
C       write(*,*)'qsel=',qsel
C       write(*,*)'hgrj=',hgrj
       call SLAKIN(AOLD(1:IENS,1:IENS,IES),
     &        pgrnew,planew,GRADDET(1:3,IEES,IES),LAPLDET(IEES,IES))
C       write(*,*)'LAPLDET,GRADDET=',LAPLDET(IEES,IES),
C     &           GRADDET(1:3,IEES,IES)
       hgrd(1:3)=GRADDET(1:3,IEES,IES)/qsel
       hlad=LAPLDET(IEES,IES)/qsel
       hkin = hlad+hlaj+2.0_dp*dot_product(hgrd,hgrj)
       velen(IE) = 0.5_dp*sum ((hgrd(1:3)+hgrj(1:3))**2)
       kinen(IE) = -0.5_dp*hkin
C       write(*,*)'IE,kinen=',IE,kinen(IE)
      end subroutine KINENERGY
C-----------------------------------------------------------------------
      subroutine DENSITY
C   Radial density RHORAD is a function of distance s from nucleus and
C   normalized to 1=sum_s 4*PI*s**2 ds RHORAD(s). It is discretized
C   in units of DRHO with NRHO sections. Values below DRHO are added
C   to first unit and those above NRHO*DRHO added to last unit.
       integer    :: j
       real(dp)   :: s,h
       RHORAD=0.0_dp
       h=4.0_dp*PI*DRHO
       s=max (sqrt(sum (RKENEW(1:3,IKM(IE),IE)**2)),DRHO+EMACH)
       s=min (s,NRHO*DRHO)
C       if (s > LATCON) write(*,*)'s=',s
       j=int (s/DRHO)
       RHORAD(j,IE)=1/h/s**2
      end subroutine DENSITY
C-----------------------------------------------------------------------
      subroutine LOCAL_DENSITY
       integer                :: i
       integer,dimension(3)   :: j
       real(dp)               :: lc
       real(dp),dimension(3)  :: r
       lc = 0.0_dp
       if (MCSTEP) then
        r(1:3) = RENEW(1:3,IE)/SULATCON
       else
        r(1:3) = RE(1:3,IE)/SULATCON
       end if
       call FOLDTOWSC(r(1:3))
C       j(1:3) = 1 + int ((0.5+r(1:3))*NDEN) ! old
       j(1:3) = int ((0.5+r(1:3)+EMACH)*NDEN)
       do i=1,3
C        if (j(i) == (NDEN+1)) j(i)=1 ! old
        if (j(i) == 0) j(i)=NDEN
       end do
       lc = (dble(NDEN)/SULATCON)**3
       AVLOCDEN(j(1),j(2),j(3))=AVLOCDEN(j(1),j(2),j(3)) + lc
C       AVLOCDEN(j(1),j(2),j(3))=AVLOCDEN(j(1),j(2),j(3))*
C     &  dble(IMC)/dble(IMC+1)+lc/dble(IMC+1)
C       write(44,*)j,IE,lc
C       write(44,*)IMC,AVLOCDEN(j(1),j(2),j(3))
      end subroutine LOCAL_DENSITY
C-----------------------------------------------------------------------
      subroutine FOURIER_COS(NDEN,rhor,rhok)
C  Discrete 3D Fourier transformation of periodic function f(n1,n2,n3)
C  on -1/2<=x_i<+1/2, i=1,2,3 with x_i=(n_i-1)/NDEN-0.5, n_i=1,...,NDEN
C  and coefficients f_k1,k2,k3. f(x) must be even, f(x)=f(-x), x=0
C  corresponds to n=NDEN/2+1.
C  In 1D it reads: f(k):=sum_{n=1,N}cos(2*PI(k-1)((n-1)/N-0.5))f(n)/NDEN
C  In the inverse transformation the factor 1/NDEN is absent, otherwise
C  it is the same transformation.
       real(dp),parameter  :: PI=3.1415927
       integer,intent(in)  :: NDEN
       real(dp),intent(in),dimension(NDEN,NDEN,NDEN)  :: rhor
       real(dp),intent(out),dimension(NDEN,NDEN,NDEN) :: rhok
C
       integer                               :: n1,n2,n3,k1,k2,k3
       real(dp)                              :: an
       real(dp),dimension(NDEN,NDEN)         :: hdcos
       real(dp),dimension(NDEN,NDEN,NDEN)    :: hhh,h1h
C
       an = dble(NDEN)
       hhh = rhor
       do k1=1,NDEN
        do n1=1,NDEN
C         hdcos(k1,n1)=1.0_dp/an*dcos(2*PI*(k1-1)*((n1-1)/an-0.5_dp)) ! up to 3.4.2011
         hdcos(k1,n1)=1.0_dp/an*dcos(2*PI*(k1-1)*(n1/an-0.5_dp)) ! beginning from 3.4.2001
        end do
       end do
C  outer loop over reciprocal space
       do k1=1,NDEN
        do k2=1,NDEN
         do k3=1,NDEN
C  inner loops over direct space
          rhok(k1,k2,k3) = 0._dp
          do n3=1,NDEN
           h1h(k1,k2,n3) = 0._dp
           do n2=1,NDEN
            h1h(k1,k2,n3) = h1h(k1,k2,n3) + hdcos(k2,n2)*
     &       dot_product (hdcos(k1,1:NDEN),hhh(1:NDEN,n2,n3))
           end do
           rhok(k1,k2,k3) = rhok(k1,k2,k3) + hdcos(k3,n3)*h1h(k1,k2,n3)
          end do
C  end of inner loops
         end do
        end do
       end do
      end subroutine FOURIER_COS
C-----------------------------------------------------------------------
      subroutine FOURIER_QUADRATURE(la,n,rhor,ewr,mpc,dicn)
C  Finite size correction:
C  Calculates model-periodic-Coulomb interaction "mpc" via dicrete
C  Fourier transformation. Needs 3D density "den", lattice type "la",
C  and dimension "n" for every cartesian direction.
C  Density "den" is sampled during MC run.
C  Number of electrons "dicn" is used for check.
C  Ewald potential "ewr" is stored in a special array at the beginning
C  of the main program.
C  n=even! dimension of spacing
       integer,intent(in)                      :: la,n
       real(dp),intent(in),dimension(n,n,n)    :: rhor,ewr
       real(dp),intent(out)                    :: mpc,dicn
C
       integer                   :: nq1,nq2,nq3,n1,n2,n3,m1,m2,m3
       integer                   :: nd2,nsign
       real(dp)                  :: dn,h,hh,hmpc
       real(dp),dimension(n,n,n) :: rhok,ewk,hr,hhr
       write(*,*)'start Fourier quadrature for finite size correction'
       nd2 = n/2
C  Symmetrize the density
       do n1=1,n
        m1=n1-1
        if(m1 == 0) m1=n
        do n2=1,n
         m2=n2-1
         if(m2 == 0) m2=n
         do n3=1,n
          m3=n3-1
          if(m3 == 0) m3=n
          hhr(n1,n2,n3)=0.5_dp*(rhor(m1,m2,m3)+
     &                          rhor(n-n1+1,n-n2+1,n-n3+1))
         end do
        end do
       end do
       call FOURIER_COS(n,hhr,rhok)
       dicn = n**3*rhok(1,1,1)*(SULATCON/dble(n))**3
       do n1=1,n
        do n2=1,n
         do n3=1,n
          hh = (n1-nd2)**2 + (n2-nd2)**2 + (n3-nd2)**2
          hh = dsqrt(hh)/dble(n)
          if (hh == 0._dp) then
C           hr(n1,n2,n3) = 0.0_dp ! actual
           hr(n1,n2,n3) = VMAD
          else
           hr(n1,n2,n3) = ewr(n1,n2,n3) - 1._dp/hh
          end if
         end do
        end do
       end do
       call FOURIER_COS(n,hr,ewk)
C       hmpc = rhok(1,1,1)**2*ewk(1,1,1) !following commented lines are for continuous space integration
C       do nq3=2,n
C        hmpc=hmpc+3/4._dp*rhok(1,1,nq3)**2*ewk(1,1,nq3)
C       end do
C       do nq2=2,n
C        do nq3=2,n
C         hmpc=hmpc+3/16._dp*rhok(1,nq2,nq3)**2*ewk(1,nq2,nq3)
C        end do
C       end do
C       do nq1=2,n
C        do nq2=2,n
C         do nq3=2,n
C          hmpc=hmpc+1/64._dp*rhok(nq1,nq2,nq3)**2*ewk(nq1,nq2,nq3)
C         end do
C        end do
C       end do
       hmpc = 0.0_dp
       do nq1=1,n ! discrete space integration
        do nq2=1,n
         do nq3=1,n
          nsign=(-1)**(nq1+nq2+nq3+1)
          hmpc=hmpc+rhok(nq1,nq2,nq3)**2*ewk(nq1,nq2,nq3)*nsign
         end do
        end do
       end do
       mpc = 0.5*hmpc*(SULATCON)**5/dble(NE)
      end subroutine FOURIER_QUADRATURE
C-----------------------------------------------------------------------
      subroutine QUADRATURE_SCWSC(la,n,mpc,dicn)
C   Calculates 6 dimensional integral of
C   rho(r1)*rho(r2)*(V_ew(r1-r2)-1/|r1-r2|_{min})
       integer,intent(in)     :: la,n
       real(dp),intent(out)   :: mpc,dicn
C
       integer                :: m1,m2,m3,n1,n2,n3,i,i1,i2,i3
       integer,dimension(3,2) :: s
       real(dp),dimension(3,2):: w
       integer,dimension(3)   :: ndf
       real(dp)               :: d1,d2,dn,hmpc,p
       real(dp),dimension(4)  :: hd
       mpc = 0._dp
       dn = dble(n)
       dicn = 0._dp
       write(*,*)'start direct quadrature for finite size correction'
       do m1=1,n
C        write(*,*)'m1,AVLOCDEN=',m1,AVLOCDEN(m1,4,4)
        do m2=1,n
C        write(*,*)'m2=',m2
         do m3=1,n
          d1 = AVLOCDEN(m1,m2,m3)/dble(IMC)
C          d1= 162/SULATCON**3
          dicn = dicn + d1
C         write(*,*)'m3=',m3
          do n1=1,n
C          write(*,*)'n1=',n1
           do n2=1,n
            do n3=1,n
C  Index to coordinate: r = -0.5_dp + dble(m)/dn,
C  left border is outside the cell.
             hd(1) = dble(m1-n1)/dn
             hd(2) = dble(m2-n2)/dn
             hd(3) = dble(m3-n3)/dn
             d2 = AVLOCDEN(n1,n2,n3)/dble(IMC)
C             d2= 162/SULATCON**3
             call FOLDTOWSC(hd(1:3))
             ndf(1:3) = int((hd(1:3)+0.5_dp+EMACH)*n)
             do i=1,3
              if (ndf(i) == 0) ndf(i)=n
             end do
             hd(4) = dsqrt( sum (hd(1:3)**2))
C             if (hd(4) <= 0.5_dp/dn) then ! actual
C              hmpc = 0.0_dp ! actual
             if (hd(4) <= 0.01_dp/dn) then
              hmpc=VMAD*d1*d2
             else
              VEW_CAL = EWALD_DEN_ARR(ndf(1),ndf(2),ndf(3))
              hmpc = d1*d2*(VEW_CAL-1/hd(4))
             end if
             mpc = mpc + hmpc
            end do
           end do
          end do
         end do
        end do
       end do
       dicn = dicn*(SULATCON/dn)**3
C  Divide by NE to obtain correction per electron
       mpc = 0.5_dp*mpc*(SULATCON/dn)**6/SULATCON/dble(NE)
      end subroutine QUADRATURE_SCWSC
C-----------------------------------------------------------------------
      end module observables
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
