C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
C    look for changes in ORBDERSPLINE with EMACH
       module orbital_hf
C   Lithium solid version
       use highlevel
       use midlevel
       use lattice_Lisolid
C
       implicit none
C   Introduces constants for HF effective potential orbitals
C   from [Sarsa,Galvez,Buendia].
C   Calculates single electron wavefunctions and its derivatives.
C   Slater parameter SLAP compress wavefunction (1s,2s only)
C   according to exp(zr)-->exp(zr/SLAP).
C
C   MPHI=number of Slater functions
C   NORB=number of orbitals per nucleus
C   NKNORB=total number of orbitals =NK*NORB
C   NELORB(NEMAX)=maps the electron index onto the set of orbitals
C   PHIC(4,MAXSPL,0:2,NORB)=Spline coefficients; 1st index counts the
C   coeffient's order, 2nd index counts the position on a radial
C   Herman-Skillman mesh, 3rd index counts zeroth, first, and second
C   derivative, and 4th index counts the number of orbital
C   HIHS(NK),HRHS=Herman-Skillman index and radial position of
C   actual electron IE referring to each nucleus
       integer,parameter,public          :: MPHI=7,NORB=2,MAXSPL=101
       integer,parameter,public          :: NKNORB=NK*NORB
       integer,dimension(NE),public      :: NELORB
       integer,dimension(NE,NK),public      :: HIHS,HIHSOLD
       real(dp),dimension(NE,NK),public     :: HRHS,HRHSOLD
       real(dp),public                   :: SLAP0,SLAPD
       real(dp),public,dimension(NORB)   :: SLAP
       real(dp),public,dimension(NKNORB) :: PSINEW
       real(dp),private,dimension(4,MAXSPL,0:2,NORB)  :: PHIC
       real(dp),public,dimension(NK,NORB)             :: WPHI,WHYPHI
C   Internal quantities
       integer,parameter,private                 :: mnphi=2,rint=10
       real(dp),parameter,private                :: minrhs=0.0_dp
       integer,private                                :: maxi
       integer,private,dimension(MPHI)                :: phinp,fak
       real(dp),private                               :: maxrhs,drhs
       real(dp),private,dimension(MPHI)          :: phizeta,norm
       real(dp),private,dimension(MPHI,NORB)     :: phiaa
       character(40), public                     :: ORBNAME
       data phinp /2*1,4*2,3/
       data phizeta / 0.72089388_dp, 2.61691643_dp, 0.69257443_dp,
     &  1.37137558_dp, 3.97864549_dp, 13.52900016_dp, 19.30801440_dp/
       data phiaa(1:MPHI,1)/ -0.12220686_dp, 1.11273225_dp,
     &  0.04125378_dp, 0.09306499_dp, -0.10260021_dp, -0.00034191_dp,
     &  0.00021963_dp /
       data phiaa(1:MPHI,2)/ 0.47750469_dp, 0.11140449_dp,
     &  -1.25954273_dp, -0.18475003_dp, -0.02736293_dp, -0.00025064_dp,
     &  0.00057962_dp /
C   Specify routine names
       public :: INITORB,INITORB3,INITORB4,ORBWAV,ORBDER,ORBSPLINE
       public :: ORBDERSPLINE,HSINDEXV,WAVETEST,LIDIMER
      contains
C-----------------------------------------------------------------------
       subroutine INITORB
C  1. Maps the electron index onto the set of orbitals through the
C     index array i=NELORB(k), i=electron index, i=orbital index via
C     PSINEW(i) depending only on the orbital index i.
C     For Li solid:
C      up-spin electron IE=1,NK;            IEES=IE occupies 1st orbital
C      up-spin electron IE=NK+1,3NK/2;      IEES=IE occupies 2nd orbital
C      down-spin electron IE=3NK/2+1,5NK/2; IEES=IE-NES(1) occ. 1st orb.
C      down-spin electron IE=5NK/2+1,3NK,   IEES=IE-NES(1) occ. 2nd orb.
C     Electrons are distributed upon the nuclei such that the 2nd orbital
C     is occupied for spin IES=1 at the 1st half of the nuclei, i.e.
C     those at integer corrdinates, say at the corner atoms, and for
C     spin IES=2 at the 2nd half of the nuclei, i.e. those at
C     half-integer coordinates and  accordingly at the centers of
C     the bcc nuclear lattice. Thus, each nucleus has 3 electrons with
C     spin-up at the corners and spin-down at the centers for the 2s
C     electron.
C  2. Calculates the factorial, i.e. (2*phinp)! and normalization
C     coefficients.
C  3. Initializes the one-electron wave function by calculating or
C     reading the spline coefficients of the tabulated waves
C  4. Calculates initial wavefunction matrix PSIMAT(i,k,s) with
C     i=orbital index, k= electron index, s= spin index
        integer,parameter                    :: nk2=NK/2
        integer                              :: j,jj
        integer,dimension(NE)                :: hnelorb
        data hnelorb(1:NK)/NK*1/
        data hnelorb(NK+1:NK+nk2)/nk2*2/
        data hnelorb(NK+nk2+1:2*NK+nk2)/NK*1/
        data hnelorb(2*NK+nk2+1:3*NK)/nk2*2/
C  1. Index array to associate each electron with a wavefunction
        NELORB(1:NE)=hnelorb(1:NE)
C  2. The factorial (2*phinp)! and normalization
        fak=1
        do j=1,MPHI
         do jj=1,2*phinp(j)-1
          fak(j)=fak(j)*(jj+1)
         end do
        end do
        norm=(2.0_dp*phizeta)**(phinp+0.5_dp)/dsqrt(dble(fak))
        ORBNAME="Compute Slater orbitals                 "
C        ORBNAME="Read Slater orbitals from spline file   "
        PHIC = 0.0_dp
        select case (ORBNAME)
         case("Compute Slater orbitals                 ")
           write(*,*)'Compute Slater orbitals directly without',
     &               'interpolation'
           write(*,*)'Slater parameters, SLAP(1:2)='
           read (*,*) (SLAP(j),j=1,NORB)
           write(*,*) (SLAP(j),j=1,NORB)
         case("Read Slater orbitals from spline file   ")
           write(*,*)'Use spline interpolation for orbitals'
           call INITORB3
         case default
           write(*,*)'No method for determination of orbitals!'
           stop
        end select
        call INITORB4
       end subroutine INITORB
C-----------------------------------------------------------------------
       subroutine INITORB3
C  3. Read orbital wave functions  from
C     file with cubic splines of Li(1s,2s) states
        real(dp),parameter  :: fak1=1._dp,fak3=1._dp/6._dp,fak2=0.5_dp
        integer                              :: j,jj
        real(dp),dimension(4,MAXSPL,3*NORB)  :: hphic
        open(35,file='PSILIDAT.DAT',status='OLD')
C        open(35,file='PHALIDAT.DAT',status='OLD')
        read (35,*) maxi,maxrhs
        if (maxi > MAXSPL) then
          write (*,*) '!!! too many spline cofficients !!!'
          write (*,*) maxi,'>',MAXSPL,' reset the limit !'
          maxi = MAXSPL
        end if
        read (35,*) (SLAP(j),j=1,NORB)
        write(*,*)'Slater parameters as tabulated in file=',
     &             (SLAP(j),j=1,NORB)
        j = 1
        jj = 1
1000    continue
        read (35,*,end=999) hphic(1,j,jj),hphic(2,j,jj),
     &                        hphic(3,j,jj),hphic(4,j,jj)
        j = j+1
        if (j > maxi) then
         j = 1
         jj = jj+1
        end if
        if (jj <= 3*NORB) go to 1000
999     continue
        close(35)
        jj = jj-1
        if (jj == 3*NORB) then
          write (39,*) 'Phi: oK, number OK !'
        else
          write (39,*) 'Phi: number not OK : ',jj,j,'<->',3*NORB,maxi
        end if
        drhs = (maxrhs-minrhs)/dble((2**(maxi/rint)-1)*rint)
C  Spline coefficients are transferred and scaled. First three (j=0,2)
C  blocks of length=maxi refer to the 0th, 1st, and 2nd derivative of
C  1s Li atomic orbital; second three blocks refer to the 2s orbital.
        do jj=1,NORB
         do j=0,2
          PHIC(1:4,1:maxi,j,jj) = hphic(1:4,1:maxi,j+(3*jj-2))
         end do
        end do
        PHIC(1:4,maxi+1:MAXSPL,0:2,1:NORB) = 0.0_dp
        PHIC(4,1:maxi,0:2,1:NORB) = PHIC(4,1:maxi,0:2,1:NORB)*fak1*fak3
        PHIC(3,1:maxi,0:2,1:NORB) = PHIC(3,1:maxi,0:2,1:NORB)*fak1*fak2
        PHIC(2,1:maxi,0:2,1:NORB) = PHIC(2,1:maxi,0:2,1:NORB)*fak1
        PHIC(1,1:maxi,0:2,1:NORB) = PHIC(1,1:maxi,0:2,1:NORB)*fak1
        write(39,*)'Phi: spline scaled'
       end subroutine INITORB3
C-----------------------------------------------------------------------
       subroutine INITORB4
C  4. The matrix psi_i(k) of the wavefunction is determined.
C     In the columns the index runs over the orbitals i for a
C     fixed electron index k.
C     But one has to discriminate between both spin directions.
C     One begins with spin-up electrons counting them
C     up to the number of electrons NES(1) with this spin.
C     Subsequently follow the spin-down electrons
C     in a new matrix up to NES(2). Take care that the orbital
C     index is correctly associated with the electron index.
C     So we associate for the Li solid the k=1,...,NE=3*NK electrons
C     with the two orbitals at each nucleus
C     in the pair (orbital[nucleus],electron) by
C                           (1[1],1),..........,(1[1],3*NK/2)
C                           .................................
C                           (1[NK],1),.........,(1[NK],3*NK/2)
C                           (2[NK/2+1],1),.....,(2[NK/2+1],3*NK/2)
C                           .................................
C                           (2[NK],1),.........,(2[NK],3*NK/2])
C     for the first matrix, i.e. the spin-up matrix, and by
C                           (1[1],3*NK/2+1),........,(1[1],3*NK)
C                           .................................
C                           (1[NK],3*NK/2+1),.......,(1[NK],3*NK)
C                           (2[NK/2+1],3*NK/2+1),...,(2[NK/2+1],3*NK)
C                           .................................
C                           (2[NK],3*NK/2+1),.......,(2[NK],3*NK)
C     for the second matrix, i.e. the spin-down matrix.
        integer                              :: j,jj,hies,hi,ii,hk
         do j=1,NE
          hies=1
          if (j > NES(1)) hies=2
          hi=(hies-1)*NES(1)
          hk=(hies-1)*NK/2
          select case (ORBNAME)
           case("Compute Slater orbitals                 ")
            call ORBWAV(RKE(4,1:NK,j),WPHI)
           case("Read Slater orbitals from spline file   ")
            call ORBSPLINE(RKE(4,1:NK,j),WPHI,HRHS(j,1:NK),HIHS(j,1:NK))
            HRHSOLD=HRHS
            HIHSOLD=HIHS
           case default
            stop
           end select
          do ii=1,NK
           PSIMAT(ii,j-hi,hies) = WPHI(ii,NELORB(ii))
          end do
           call LIDIMER(WPHI,WHYPHI) !for hybrid states
          do ii=NK+1,3*NK/2
          PSIMAT(ii,j-hi,hies) = WHYPHI(ii-NK+hk,NELORB(ii)) !for hybrid states
C           PSIMAT(ii,j-hi,hies) = WPHI(ii-NK+hk,NELORB(ii))
          end do
         end do
       end subroutine INITORB4
C-----------------------------------------------------------------------
       subroutine ORBWAV(rke,wphi)
        real(kind=dp),intent(in),dimension(NK)       :: rke
        real(kind=dp),intent(out),dimension(NK,NORB) :: wphi
C   Calculates the Hartree atomic wavefunctions of Lithium
C   according to [Sarsa,Galvez,Buendia] with respect to each nucleus.
C   Needs more (???%) CPU time than spline interpolation.
        integer             :: j,js
        real(kind=dp)       :: s
        wphi = 0.0_dp
        do j=1,NK
         do js=1,MPHI
           wphi(j,1:NORB)=wphi(j,1:NORB)+phiaa(js,1:NORB)*
     &                         rke(j)**(phinp(js)-1)*norm(js)*
     &                         dexp(-phizeta(js)*rke(j)/SLAP(1:NORB))
         end do
        end do
       end subroutine ORBWAV
C-----------------------------------------------------------------------
       subroutine ORBDER(rke,wphi1,wphi2)
C   Gradient, laplacian, and wave function from HF effective potential
C   atomic wavefunction via [Sarsa,Galvez,Buendia].
        real(dp),intent(in),dimension(4,NK)       :: rke
        real(dp),intent(out),dimension(NK,NORB)   :: wphi2
        real(dp),intent(out),dimension(3,NK,NORB) :: wphi1
C
        integer                        :: i,j,js
        real(dp),dimension(NORB)       :: psid1,psid2
        real(dp),dimension(MPHI,NORB)  :: p1,p2,p3
        real(kind=dp)                   :: s
        do j=1,NK
         s=rke(4,j)
         if (s < EMACH) s=KINEMACH
         do js=1,MPHI
          p1(js,1:NORB)=phiaa(js,1:NORB)*norm(js)*
     &                      dexp(-phizeta(js)*s/SLAP(1:NORB))
         end do
C  first derivative of wave function
         psid1 = 0.0_dp
         do js=1,MPHI
          p2(js,1:NORB)=p1(js,1:NORB)*phizeta(js)/SLAP(1:NORB)*
     &                                               s**(phinp(js)-1)
          p3(js,1:NORB)=p1(js,1:NORB)*(phinp(js)-1)*s**(phinp(js)-2)
          psid1(1:NORB)=psid1(1:NORB)+p3(js,1:NORB)-p2(js,1:NORB)
         end do
C  second derivative of wave function
         psid2 = 0.0_dp
         do js=1,MPHI
          psid2(1:NORB)=psid2(1:NORB) +
     &                  p2(js,1:NORB)*phizeta(js)/SLAP(1:NORB) -
     &           2.0_dp*p3(js,1:NORB)*phizeta(js)/SLAP(1:NORB) +
     &                  p3(js,1:NORB)*(phinp(js)-2)/s
         end do
         do i=1,3
          wphi1(i,j,1:NORB)=rke(i,j)/s*psid1(1:NORB) !gradient of wave function
         end do
         wphi2(j,1:NORB) = psid2(1:NORB) + 2.0_dp/s*psid1(1:NORB) !Laplacian
        end do
       end subroutine ORBDER
C-----------------------------------------------------------------------
       subroutine ORBSPLINE(rke,wphi,rd,ir)
C  Calculates the orbital wavefunction wphi with spline interpolation
C  from spline coefficients PHIC on a Herman-Skilman mesh
C  with points "ir" and deviations "rd" from these. The association
C  of a space point "rke" with (ir,rd) is calculated here, too.
C  rke(1:NK) distance electron IE from all nuclei
C  ir(1:NK) mesh indices at electron distances
        real(dp),intent(in),dimension(NK)       :: rke
        integer,intent(out),dimension(NK)       :: ir
        real(dp),intent(out),dimension(NK)      :: rd
        real(dp),intent(out),dimension(NK,NORB) :: wphi
C  Internal variables
        integer                             :: j,jp
        real(dp),dimension(NK)              :: drh
C  Determine the index ir for the new position of electron IE with
C  respect to a nucleus within a Herman-Skillman mesh
C  rke(1:NK) = RKENEW(4,1:NK,IE), and do that for every nucleus.
        call HSINDEXV(rint,drhs,NK,rke,rd,drh,ir)
        do j=1,NK
         if (ir(j) > maxi) then
          ir(j) = maxi
          rd(j) = 0.0_dp
         end if
        end do
C  Calculate atomic wavefunction by the spline at index ir(j) of the
C  mesh from the center of nucleus j for a distance rd(j) from the
C  neighboring mesh point below with help of the Horner scheme
        do jp = 1,NORB
         do j = 1,NK
          wphi(j,jp) = ((PHIC(4,ir(j),0,jp)*rd(j)  +
     &                   PHIC(3,ir(j),0,jp))*rd(j) +
     &                   PHIC(2,ir(j),0,jp))*rd(j) +
     &                   PHIC(1,ir(j),0,jp)
         end do
        end do
       end subroutine ORBSPLINE
C-----------------------------------------------------------------------
       subroutine ORBDERSPLINE(rke,rd,ir,wphi1,wphi2)
C  Calculates 1st and 2nd derivative of the orbital wavefunction
C  from the spline interpolation with coefficients PHIC
        integer,intent(in),dimension(NK)          :: ir
        real(dp),intent(in),dimension(NK)         :: rd
        real(dp),intent(in),dimension(4,NK)       :: rke
        real(dp),intent(out),dimension(NK,NORB)   :: wphi2
        real(dp),intent(out),dimension(3,NK,NORB) :: wphi1
C  Internal variables
        integer      :: j,jj,jp
        real (dp)    :: s
        real(dp),dimension(NK,NORB) :: h
C  The 1st radial derivative
        do jp = 1,NORB
         do j = 1,NK
                h(j,jp) = ((PHIC(4,ir(j),1,jp)*rd(j)  +
     &                      PHIC(3,ir(j),1,jp))*rd(j) +
     &                      PHIC(2,ir(j),1,jp))*rd(j) +
     &                      PHIC(1,ir(j),1,jp)
         end do
        end do
C  The 2nd radial derivative for the Laplacian
        do jp = 1,NORB
         do j = 1,NK
          s = rke(4,j)
          if (s < EMACH) s=KINEMACH
          wphi2(j,jp) = ((PHIC(4,ir(j),2,jp)*rd(j)  +
     &                    PHIC(3,ir(j),2,jp))*rd(j) +
     &                    PHIC(2,ir(j),2,jp))*rd(j) +
     &                    PHIC(1,ir(j),2,jp) +
     &                    2._dp/s*h(j,jp)
         end do
        end do
C  Multiplication by the radial unit vector for the gradient
        do jp = 1,NORB
         do jj=1,3
          wphi1(jj,1:NK,jp) = h(1:NK,jp)*
     &             rke(jj,1:NK)/(rke(4,1:NK)+EMACH)
         end do
        end do
        end subroutine ORBDERSPLINE
C-----------------------------------------------------------------------
       subroutine HSINDEXV(rint,dr,n,rr,rd,drh,ir)
C  rint,dr= mesh constants
C  n= number of vector components
C  rr= radial distance for each component
C  ir= mesh index at radial distance, see below the mesh
C  rd= remainder of rr when subtracting its integer portion
C  at spacing with drh
C  drh= highest attained value of spacing between two mesh points
        integer,intent(in)   :: rint,n
        real(dp),intent(in)  :: dr
        real(dp),intent(in),dimension(n)  :: rr
        integer,intent(out),dimension(n)  :: ir
        real(dp),intent(out),dimension(n) :: rd,drh
C  internal variables
        integer                           :: ia
        integer,dimension(n)              :: i,ni,i1
Cunder construction
C       geg: RINT,DR,MN,N(<=MN),RR(MN)
C          und Hilfsvar.: I(MN),NI(MN),I1(MN) (Output)
C       ges: IR(.) mit  R(IR(.)) <= RR(.) < R(IR(.)+1)
C            im Herman-Skilman-Raster mit MINR = R(1) = 0,
C            minimaler Schrittweite  DR = R(2)-R(1)  und
C            Anzahl von Intervallen mit gleicher Schrittweite RINT
C            RD(.) = RR(.)-R(IR(.))
C            DRH(.) = R(IR(.)+1)-R(IR(.))
C
C  Index i in an equidistant mesh with dr*rint spacing, first index =1
        i = int(rr/(dr*rint))+1
C   ni=int(Log2 (I)) or i=2**ni
        do ia=1,n
         ni(ia) = 0
         i1(ia) = i(ia)
         i(ia) = 1
222      continue
         i1(ia) = i1(ia)/2
         if (i1(ia) > 0) then
          ni(ia) = ni(ia)+1
          i(ia) = i(ia)*2
          goto 222
         end if
        end do
C  The mesh:
C  Index ir in an equidistant mesh with dr*i spacing,
C  starting with 1 extending to rint at rr=dr*rint with dr spacing,
C  continuing with rint+1 up to rint*2 at rr=dr*rint*3, spacing dr*2,
C  then with rint*2+1 up to rint*3 at rr=dr*rint*7, spacing dr*4
C  etc. with rint*ni+1 up to rint*(ni+1) at rr=dr*rint*(2**(ni+1)-1),
C  spacing dr*2**ni=dr*i.
C  Caution: no upper limit has been yet defined,
C  has to be externally set.
        i1 = int((rr-dr*rint*(i-1))/(dr*i))
        ir = i1 + 1 + ni*rint
C  Relations can be written as: i1=mod(ir-1,rint), ni=(ir-1)/rint,
C  i=2**ni, rr=dr*dble((i1+rint)*i-rint)
        drh = dr*i
        rd = rr - dr*dble((i1+rint)*i-rint)
      end subroutine HSINDEXV
C-----------------------------------------------------------------------
      subroutine WAVETEST
       integer                               :: n
       real(dp)                              :: h
       integer,dimension(NK)                 :: iir
       real(dp),dimension(NK)                :: rrd
       real(dp),dimension(4,NK)              :: rr
       real(dp),dimension(3,NK,NORB)         :: pgr
       real(dp),dimension(NK,NORB)           :: pla
       real(dp)     :: f1,f2,fgr1,fgr2,fla1,fla2
C  Output of cubic spline wavefunction and its derivatives
      open(unit=61,file='wave1.DAT',status='unknown')
      open(unit=62,file='wave2.DAT',status='unknown')
      open(unit=63,file='wave_grad1.DAT',status='unknown')
      open(unit=64,file='wave_grad2.DAT',status='unknown')
      open(unit=65,file='wave_laplace1.DAT',status='unknown')
      open(unit=66,file='wave_laplace2.DAT',status='unknown')
      rr = 0._dp
      do n= 1,1000
       rr(4,1:NK) = 0.00001 + dble(n)*0.01_dp
       rr(1,1:NK) = rr(4,1:NK)
       h = rr(4,1)
       call ORBSPLINE(rr(4,1:NK),WPHI,rrd,iir)
       call ORBDERSPLINE(rr(1:4,1:NK),rrd,iir,pgr,pla)
C       call ORBWAV(rr(4,1:NK),WPHI)
C       call ORBDER(rr(1:4,1:NK),WPHI1,WPHI2)
       f1 = WPHI(1,1)
       f2 = WPHI(1,2)
       fgr1 = pgr(1,1,1)
       fgr2 = pgr(1,1,2)
       fla1 = pla(1,1)
       fla2 = pla(1,2)
       write(61,*) h,f1
       write(62,*) h,f2
       write(63,*) h,fgr1
       write(64,*) h,fgr2
       write(65,*) h,fla1
       write(66,*) h,fla2
      end do
      close(61)
      close(62)
      close(63)
      close(64)
      close(65)
      close(66)
      end subroutine WAVETEST
C-----------------------------------------------------------------------
      subroutine LIDIMER(w,why)
C   For 4x4x4 supercell only.
C   Associates for the 2s states hybrid wavefunctions on both nuclei
C   of the bcc unitcell instead of using only atomic wavefunctions
C   on the 0.5 positions. Has to be called after calling ORBSPLINE and
C   ORBDERSPLINE.
       real(dp),dimension(NK,NORB),intent(in)  :: w
       real(dp),dimension(NK,NORB),intent(out) :: why
C
       integer :: i,k,ik
       integer,dimension(NK/2,NK) :: lcao
       why = w
       ik=0
       do i=1,66  !non-vanishing contributions real part
        if (GCUTR(i) == 0) cycle
        if ((i == 60) .or. (i==61) .or. (i==65))cycle
        ik=ik+1
        why(ik,2) = dot_product (RLCAO(i,1:NK),w(1:NK,2))  
       end do
       do i=1,57   !non-vanishing contributions imag. part
        if (GCUTI(i) == 0) cycle
        ik=ik+1
        why(ik,2) = dot_product (ILCAO(i,1:NK),w(1:NK,2))
       end do
C  7 states are still missing beyond number=57. 
C  From the 24 states of g**2= (PI/2)**2*6 only 6 are available 
C  independently as real and imaginary part. We use state 58,59 62,63, 
C  64,66 for the real partand the sum of all for imag. part. 
C  real part: (-2-1-1),(-2-11),(-1-2-1),(-1-21),(-1-1-2),(-11-2)
C  imag. part: sum of those above
       ik=ik+1
       why(ik,2) = 0._dp
       do i=58,81
        why(ik,2) = why(ik,2) + dot_product (ILCAO(i,1:NK),w(1:NK,2))
       end do
       if (ik /= NK/2) then 
        write (*,*)'number of states not equal NK/2:ik=',ik
        stop
       end if
       do i=1,NK/2
        why(NK/2+i,2) = why(i,2)        
       end do
      end subroutine LIDIMER
C-----------------------------------------------------------------------
      end module orbital_HF
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
