      module highlevel
       implicit none
C  Constants
       integer,parameter  :: dp=selected_real_kind(2*precision(1.0)) ! double precision
       real(dp),parameter :: PI=4._dp*datan(1._dp),EMACH=0.0000001
       integer,parameter  :: MWALK=5000    ! maximum no. of walkers
       integer,parameter  :: NTOT=40       ! 2*NTOT+1 points of support for density
       integer,parameter  :: MCMAX=20000000 ! # QMC time steps
       integer,parameter  :: NE=1          ! # electrons
       integer,parameter,dimension(2) :: NES=(/1,0/)    ! # electrons per spin
       real(dp),parameter :: LCLUSTER=2._dp*100._dp     ! box=[-LCLUSTER,+LCLUSTER]/2
       real(dp),parameter :: SIGMA=0.01_dp,TAU=SIGMA**2 ! Gaussian width, time step width
       real(dp),parameter :: DELTA=0.0001_dp ! feedback strength
       real(dp),parameter :: OMEGA=1._dp   ! oscillator frequency
C  Variables
       integer   :: MCH,MCHN   ! target # walkers, actual # walkers
       integer   :: M0,MCCON   ! global # walkers, # steps between control
       integer   :: MCC        ! intermediate step index between control
       integer   :: MC,MW,IE   ! MC,MW,IE=index of step,walker,electron
       real(dp)  :: VPOT,ENERT  ! energies: potential, total
       real(dp)  :: AVPM,AVPMC,AVMCH,AVMCHO,AVPOT,AVENERT ! averages
       real(dp)  :: VARMCH,VARPOT,VARENERT                ! variances
       real(dp)  :: FRAN,GRAN  ! random numbers: uniform, Gaussian 
       real(dp),dimension(MWALK) :: RE,RNEU ! position of walkers: old, actual
       real(dp),dimension(MWALK) :: WEIGHT  ! weight of walkers  
       real(dp),dimension(2*NTOT+1) :: XRHO,RHO,AVDEN,AVDENNEU,DENEXACT
       real(dp)  :: RHODEV     ! density deviation from average at control steps
C
      end module highlevel
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      module random
       use highlevel
       implicit none
       public :: GENRAN,GRNF,VPOTF
      contains
C-----------------------------------------------------------------------
      subroutine GENRAN(rannumb)
       real(dp),intent(out) :: rannumb
       call random_number(rannumb)
      end subroutine GENRAN
C-----------------------------------------------------------------------
      subroutine GRNF(x,y,z)
       real(dp),intent(out) :: x,y,z
C  local variables
       real(dp)             :: r1,r2,rannumb
       call GENRAN(rannumb)
       r1 = -dlog(1.0_dp-rannumb+0.000000000001_dp)
       call GENRAN(rannumb)
       r2 = 2._dp*PI*rannumb
       r1 = dsqrt(2._dp*r1)
       x  = r1*dcos(r2)
       y  = r1*dsin(r2)
       call GENRAN(rannumb)
       z  = rannumb
      end subroutine GRNF
C-----------------------------------------------------------------------
      function VPOTF(r,om)
C  VPOTF and om given in Hartree, r in Bohr
       real(dp)  :: VPOTF
       real(dp),intent(in)  :: r,om
       VPOTF = 0.5_dp*(om*r)**2
      end function VPOTF
C-----------------------------------------------------------------------
      end module random
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      program diffusion_oscillator
C
C  DQMC main program of cubic cluster with infinite walls
C
       use highlevel
       use random
       implicit none
C  local variables
       integer      :: i,k,m,m1,m2,hm1,hm2,hi1,hi2,hi3,histsum
       integer,dimension(MWALK):: hist
       real(dp)     :: rannumb,dum,rd,vp,vpn,hvp,hki,hpot,hdev,hnu
       real(dp)     :: norm,hexact
       logical      :: decis1,decis2
C  control file
       open (unit=35,file="DMC_20M_s001_d00001.OUT",
     &                                      status="unknown")
C  initialize the walkers' positions equally distributed
C  on the LCLUSTER*[-0.5,+0.5] rectangle
       M0 = 500
       MCCON = 100
       if (M0 .GT. MWALK) THEN
        write (*,*) 'Error: Array bound in RE(MWALK), MWALK= ',
     &              MWALK,' > 10000'
       end if
C  Calculate the initial walker positions, here a triangular
C    distribution on [-1,+1] for LCLUSTER=2.
       hist = 0
       do m1=1,M0
         call GENRAN(rannumb)
         hnu = (rannumb-0.5_dp)
         RE(m1)=sign(1._dp,hnu)*(1-dsqrt(1-2._dp*dabs(hnu)))*LCLUSTER/2
         write (58,*)'m1,RE(m1) =',m1,RE(m1)
C          m = 1 + nint((RE(m1)+1)*20) !show the distribution
C          hist(m) = hist(m) + 1
       end do
C  initialize a set of Gaussian distributed random numbers
       write (35,'(3(a,f12.5))')'SIGMA=',SIGMA,'  TAU= ',TAU,
     &  '  DELTA=',DELTA,'  LCLUSTER=',LCLUSTER
       AVPM = 0._dp
       AVPMC = 0._dp
       AVMCH = 0._dp
       AVPOT = 0._dp
       AVENERT = 0._dp
       AVDEN = 0._dp
       AVDENNEU = 0._dp
       MCC = 0
       MCH = M0
       do m=1,2*NTOT+1
        XRHO(m) = (dble(m-1)/dble(NTOT)-1._dp)*LCLUSTER/50._dp 
       end do
       MCHN = M0
C
C  start Monte-Carlo run
       lqmc:do MC=1,MCMAX
        MW = 0
        vp = 0._dp
C  Move walkers
        lcopies:do while (MW <= MCH-1)
          MW = MW + 1
          AVPM=0._dp
C  Let the walker try a diffusion step by Gaussian increment
          call GRNF(GRAN,dum,FRAN)
          rd = GRAN*SIGMA
          RNEU(MW) = RE(MW) + rd ! proposal for diffusion
          decis1 = (dabs(RNEU(MW)) > LCLUSTER/2._dp)
          if (decis1) then 
           WEIGHT(MW) = 0._dp ! weight for removal
          else
C  distribute according to the mean potential-energy
C  through controlling the number of walkers by the exponential,
C  use single steps below for the potential sampling
          hpot = VPOTF(RNEU(MW),OMEGA)
          vp = vp + hpot
C  source-sink probability of walkers
           WEIGHT(MW) = dexp(-(hpot-ENERT)*TAU)
          end if
        end do lcopies
C  feedback from # walkers to trial energy
        VPOT = vp/MCH 
        ENERT = VPOT - DELTA*dble(MCH-M0)/dble(M0)/TAU
        write (60,*)'MC,MCH,ENERT =',MC,MCH,ENERT
        MCHN = MCH
C  adjust # walkers from weight
        do MW=1,MCH 
          call GENRAN(rannumb)
          hm1 = min (3,int(WEIGHT(MW)+rannumb))
          if (hm1 == 0) then
            MCHN = MCHN - 1
            if (MCHN <= 0) then
              write (*,*)'zero # of walkers, MCHN= ',MCHN
              write (35,*)'zero # of walkers, MCHN= ',MCHN
              stop
            end if
            m1 = MW + 1 
            if (m1 <= MCHN+1) then
              do while ((WEIGHT(m1) == 0._dp) .and. (m1 <= MCHN)) ! remove that walker
                m1=m1+1
              end do 
              if (m1 <= MCHN+1) then
               RNEU(MW) = RNEU(m1)
               WEIGHT(MW) = WEIGHT(m1)
               do m2=m1,MCHN
                RNEU(m2) = RNEU(m2+1)
                WEIGHT(m2) = WEIGHT(m2+1)
               end do
              end if
            end if
          else if (hm1 > 1) then
            MCHN = MCHN + 1
            if (MCHN > MWALK) then
             write (*,*)'# of walkers too large, MCHN=',MCHN,'>',MWALK
             write (35,*)'# of walkers too large, MCHN=',MCHN,'>',MWALK
             stop
            end if
            do m1=MCHN,MCHN ! create a new walker at that position
             RNEU(m1) = RNEU(MW)
             WEIGHT(m1) = WEIGHT(MW)
            end do
          end if
        end do
        MCH = MCHN
        RE(1:MCH) = RNEU(1:MCH)
        MCC = MCC + 1
        AVMCH = dble(MCC-1)/dble(MCC)*AVMCH + MCH/dble(MCC)
        AVPOT = dble(MCC-1)/dble(MCC)*AVPOT + VPOT/dble(MCC)
        AVENERT = dble(MCC-1)/dble(MCC)*AVENERT + ENERT/dble(MCC)
        if (MCC == 1) then
         VARMCH = 0._dp
         VARENERT = 0._dp
        else
         VARMCH = dble(MCC-2)/dble(MCC-1)*VARMCH + 
     &       dble(MCC)/dble(MCC-1)**2*(AVMCH-MCH)**2
         VARENERT = dble(MCC-2)/dble(MCC-1)*VARENERT + 
     &       dble(MCC)/dble(MCC-1)**2*(AVENERT-ENERT)**2
        end if
C  control every MCCON'th step # of walkers, energy and average potential
        if (MC/MCCON == dble(MC)/dble(MCCON)) then 
         write (35,*) 'step MC=',MC,', # of walkers MCH=',MCH,
     &              ', energy ENERT=',ENERT,'vp=',vp
         write (35,*)'AVMCH,VARMCH=',AVMCH,VARMCH
         write (35,*)'ENERT=',ENERT,'AVENERT,VARENERT=',AVENERT,VARENERT
         write (35,*)'RHODEV=',RHODEV
         write (35,*)
        end if
C  Sample the density
        RHO = 0._dp
        do m1=1,MCH
         m = 1 + nint((RNEU(m1)*50._dp/LCLUSTER+1)*NTOT)
         if ((m<1) .or. (m>2*NTOT+1)) cycle
         RHO(m) = RHO(m) + 1._dp/dble(MCH)
        end do
        RHO = RHO*2*NTOT/LCLUSTER
        AVDENNEU = dble(MCC-1)/dble(MCC)*AVDENNEU + RHO/dble(MCC)
        if (MC/MCCON == dble(MC)/dble(MCCON)) then  ! every MCCON step
         hdev = 0._dp
         do hi3=1,2*NTOT+1
            hdev = hdev + (AVDENNEU(hi3)-AVDEN(hi3))**2
         end do
         RHODEV = dsqrt(hdev)/dble(2*NTOT+1)
         write (35,*)'RHODEV= ',RHODEV
         AVDEN = AVDENNEU
        end if
       end do lqmc
C  Do output
        write (*,*)'AVMCH,VARMCH=',AVMCH,VARMCH
        write (*,*)'AVENERT,VARENERT = ',AVENERT,VARENERT
        write (*,*)'RHODEV= ',RHODEV
        open (UNIT=36,file="density20M_s001_d00001.OUT",
     &                                         status="unknown")
        write (36,'(a,i4,a,i4,2(a,e12.5))')'# MCMAX=',MCMAX,
     &       '2*NTOT+1 = ',2*NTOT+1,' RHODEV =',RHODEV,' ENERT =',ENERT
        hdev = 0._dp
        hexact = 0._dp
        do hi3=1,2*NTOT+1  ! integrate with Trapez
           DENEXACT(hi3) = cos(PI/LCLUSTER*(hi3-NTOT-1)/dble(NTOT))
           DENEXACT(hi3) = dexp(-0.5_dp*(800._dp/LCLUSTER*(hi3-NTOT-1)/
     &         dble(NTOT))**2)
           hdev = hdev + AVDENNEU(hi3)
           hexact= hexact + DENEXACT(hi3)
        end do
        norm = hdev*LCLUSTER/dble(2*NTOT)
        hexact = hexact*LCLUSTER/dble(2*NTOT)
        DENEXACT = DENEXACT/hexact 
        AVDENNEU = AVDENNEU/norm
        write(35,*)'norm of density =',norm
        write (36,'(2a)')'# Index QMCDensity',  
     &  ' ExactDensity'
        write(36,'(3f12.5)') (XRHO(hi3),AVDENNEU(hi3),
     &                                 DENEXACT(hi3),hi3=1,2*NTOT+1)
        close (36)
        close (35)
       end program diffusion_oscillator
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
